#ifndef _LINUX_DELAY_H
#define _LINUX_DELAY_H

/*
 * Copyright (C) 1993 Linus Torvalds
 *
 * Delay routines, using a pre-computed "loops_per_jiffy" value.
 */

extern unsigned long loops_per_jiffy;

#include <asm/delay.h>

/*
 * Using udelay() for intervals greater than a few milliseconds can
 * risk overflow for high loops_per_sec (high bogomips) machines. The
 * mdelay() provides a wrapper to prevent this.  For delays greater
 * than MAX_UDELAY_MS milliseconds, the wrapper is used.  Architecture
 * specific values can be defined in asm-???/delay.h as an override.
 * The 2nd mdelay() definition ensures GCC will optimize away the 
 * while loop for the common cases where n <= MAX_UDELAY_MS  --  Paul G.
 */

#ifndef MAX_UDELAY_MS
#define MAX_UDELAY_MS	5
#endif

#ifdef notdef
#define mdelay(n) (\
	{unsigned long msec=(n); while (msec--) udelay(1000);})
#else
#define mdelay(n) (\
	(__builtin_constant_p(n) && (n)<=MAX_UDELAY_MS) ? udelay((n)*1000) : \
	({unsigned long msec=(n); while (msec--) udelay(1000);}))
#endif
#ifdef CONFIG_LOWLATENCY
/*
 * the 'preemptive' version of udelay. In some cases we want to use
 * this variant, it guarantees that preemption will happing within
 * 10 usecs (despite doing busy waiting). Not all drivers can use
 * this automatically, the driver has to be sufficiently reentrant.
 */
#define udelay_resched(n) (\
	{	int i; \
		for (i = 0; i < ((n+9)/10); i++) { \
			conditional_schedule(); \
			udelay(10); \
		} \
	})

#define mdelay_resched(n) (\
	{unsigned long msec=(n); while (msec--) udelay_resched(1000);})
#endif /* CONFIG_LOWLATENCY */

#endif /* defined(_LINUX_DELAY_H) */
